import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * This version of the TalkerJF1 example makes use of the SerialPortEventListener to avoid polling.
 * http://rxtx.qbang.org/wiki/index.php/Event_based_two_way_Communication
 */
public class TalkerJF1
{
    static int addHigh;
    static int addLow;
    static int byteSpan;

    public TalkerJF1(){super();}

    void connect ( String portName ) throws Exception
    {
        CommPortIdentifier portIdentifier = CommPortIdentifier.getPortIdentifier(portName);
        if ( portIdentifier.isCurrentlyOwned() )
        {System.out.println("Error: Port is currently in use");
        }else{
            CommPort commPort = portIdentifier.open(this.getClass().getName(),2000);
            if ( commPort instanceof SerialPort )
            {
// set timeout to 600ms (apparently the min is 100) ... @4800 baud 600ms = 288 bytes
//----------------------------------------------------------------------------------
                commPort.enableReceiveTimeout(600);
//----------------------------------------------------------------------------------

                SerialPort serialPort = (SerialPort) commPort;
                serialPort.setSerialPortParams(4800,SerialPort.DATABITS_8,SerialPort.STOPBITS_1,SerialPort.PARITY_NONE);

                InputStream in = serialPort.getInputStream();
                OutputStream out = serialPort.getOutputStream();

                (new Thread(new SerialWriter(out))).start();

                serialPort.addEventListener(new SerialReader(in));
                serialPort.notifyOnDataAvailable(true);
            }else{
                System.out.println("Error: Only serial ports are handled by this example.");
            }
        }
    }

    /**
     * Handles the input coming from the serial port. A new line character
     * is treated as the end of a block in this example.
     */
    public static class SerialReader implements SerialPortEventListener
    {
        private InputStream in;
        private byte[] buffer = new byte[1100];

        public SerialReader ( InputStream in ){this.in = in;}

        public void serialEvent(SerialPortEvent arg0) {
            int data;
            try
            {
                int len = 0;
                while ( ( data = in.read()) > -1 )
                {
//debuging          System.out.println("Receiving from MCU byte# " +(len+1)+ " decimal <" +data+ ">");
                    buffer[len++] = (byte) data;
                }

//debuging                System.out.println("Command 2s Complement = " +buffer[0]);

                StringBuffer sb = new StringBuffer();
                for (int i=0; i<len; i++)
                {
                     sb.append(String.format("%02X ", buffer[i] & 0xff) );
                }

                if(buffer[0]==0) // F: FFhex = -1 : returned 2s complement = 0 decminal
                {                // =
                    System.out.println();
                    System.out.println("8 x FF returns:");
                    System.out.println(sb.toString());
                }

                if(buffer[0]==126) // R: 81hex = -127 : returned 2s complement = 7E = 126 decminal
                {                  // =
                    System.out.println();
                    System.out.println("MCU Registers:");
                    String regs = sb.toString();
                    System.out.println(regs.substring(3));
                    System.out.println("Stack CC  B  A [IX ] [IY ] [PC ]");
                }

                 if(buffer[0]==-2) // E or M: 1hex = 1 : returned 2s complement = FE = -2 decminal
                 {                 // ======
                     if(addHigh==0){
						 System.out.println();
						 System.out.println("RAM Memory:");
						 String d = sb.toString();
						 System.out.println("      +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F");
						 for(int k=0; k<4; k++){
							 for(int j=0; j<16; j++){
								 int highi = addHigh + j*16/256 + k;
								 int lowi  = addLow  + j*16;
								 StringBuffer lab = new StringBuffer();
								 lab.append(String.format("%02X", highi) );
								 lab.append(String.format("%02X", lowi) );
								 System.out.println(lab.toString() +"  " + d.substring(3*j*16+3 +3*k*257,3*(j+1)*16+3 +3*k*257));
							 }
							 System.out.println();
						 }
						 System.out.println();
					 }

					 if(addHigh==254){
						 System.out.println("EEPROM Memory:");
						 String d = sb.toString();
						 System.out.println("      +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F");
						 for(int k=0; k<2; k++){
							 for(int j=0; j<16; j++){
								 int highi = addHigh + j*16/256 + k;
								 int lowi  = addLow  + j*16;
								 StringBuffer lab = new StringBuffer();
								 lab.append(String.format("%02X", highi) );
								 lab.append(String.format("%02X", lowi) );
								 System.out.println(lab.toString() +"  " + d.substring(3*j*16+3 +3*k*257,3*(j+1)*16+3 +3*k*257));
							 }
							 System.out.println();
						 }
				     }
                     if(addHigh==16){
						 System.out.println();
						 System.out.println("GIO Ports accessed via 96 bytes starting at 1000(hex):");
						 System.out.println("------------------------------------------------------");
						 System.out.println();
						 String d = sb.toString();
						 System.out.println("      +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F");
						 for(int j=0; j<6; j++){
							 int highi = addHigh + j*16/256;
							 int lowi  = addLow  + j*16;
							 StringBuffer lab = new StringBuffer();
							 lab.append(String.format("%02X", highi) );
							 lab.append(String.format("%02X", lowi) );
							 System.out.println(lab.toString() +"  " + d.substring(3*j*16+3,3*(j+1)*16+3));
						 }
						 System.out.println();
						 System.out.println("Selected Ports and Direction Control:");
						 System.out.println("-------------------------------------");
                         String ports = sb.toString();
                         StringBuffer sbits;
                         String bits;
                         int base;

//                       Port A (value:1000 direction:1001)
//                       ----------------------------------
                         base=1;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("Port A (1000):  " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0"));
						 sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base+1] & 0xff )));
                         bits = sbits.toString();
						 System.out.println("..DDRA (1001) direction (in=0, out=1) " +bits.replaceAll(" ", "0"));
						 System.out.println();

//                       Port B (value:1004 ...output only)
//                       ----------------------------------
                         base=5;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("Port B (1004): " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0")+ " output only");
						 System.out.println();

//                       Port C (value:1006 direction:1007)
//                       ----------------------------------
                         base=7;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("Port C (1006): " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0"));
						 sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base+1] & 0xff )));
                         bits = sbits.toString();
						 System.out.println("..DDRC (1007) direction (in=0, out=1) " +bits.replaceAll(" ", "0"));
						 System.out.println();

//                       Port D (value:1008 direction:1009)
//                       ----------------------------------
                         base=9;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("Port D (1008): " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0"));
						 sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base+1] & 0xff )));
                         bits = sbits.toString();
						 System.out.println("..DDRD (1009) direction (in=0, out=1) " +bits.replaceAll(" ", "0"));
						 System.out.println();

//                       Port F (value:1005 ...output only)
//                       ----------------------------------
                         base=6;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("Port F (1005): " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0")+ " output only");
						 System.out.println();

//                       Port G (value:1002 direction:1003)
//                       ----------------------------------
                         base=3;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("Port G (1002): " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0"));
						 sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base+1] & 0xff )));
                         bits = sbits.toString();
						 System.out.println("..DDRG (1003) direction (in=0, out=1) " +bits.replaceAll(" ", "0"));
						 System.out.println();

//                       Port E (value:100A ...input only)
//                       ---------------------------------
                         base=11;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("Port E (100A) may be used for binary input or as analog input for A/D");
						 System.out.println("---------------------------------------------------------------------");
						 System.out.println("Port E (as binary): " +ports.substring(3*base,3*base+2)+ "(hex)   " +bits.replaceAll(" ", "0")+ "  [Do not read E during A/D]");
						 System.out.println();

						 System.out.println("If Port E was used for A/D:");
						 System.out.println("---------------------------");
						 System.out.println("A write to ADCTL starts A/D conversion");
						 System.out.println();

                         base=49;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("ADCTL (1030): " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0"));
						 System.out.println("  CCF        (bit 7) = " +bits.substring(0,1)+ "        (1=Conversion complete)");
						 System.out.println("  not used   (bit 6)    .");
						 System.out.println("  SCAN       (bit 5) =   " +bits.substring(2,3)+ "      (0=do 4 samples & stop, 1=continuous)");
						 System.out.println("  MULT       (bit 4) =    " +bits.substring(3,4)+ "     (0=use single channel, 1=use 4 channels)");
						 System.out.println("  CD         (bit 3) =     " +bits.substring(4,5)+ "    (4 channels...0=use {0123}, 1=use {4567})");
						 System.out.println("  CC/CA (bits 2,1,0) =      " +bits.substring(5,8)+ " (1 channel...read from this port E pin)");
						 System.out.println();
						 System.out.println("A/D data registers");
						 System.out.println("------------------");

                         base=32;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("ADR0 (1031) data for A/D channels 0 & 4 = " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0"));

                         base=33;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("ADR1 (1032) data for A/D channels 1 & 5 = " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0"));

                         base=34;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("ADR2 (1033) data for A/D channels 2 & 6 = " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0"));

                         base=35;
                         sbits = new StringBuffer();
                         sbits.append(String.format("%8s", Integer.toBinaryString( buffer[base] & 0xff )));
						 bits = sbits.toString();
						 System.out.println("ADR3 (1034) data for A/D channels 3 & 7 = " +ports.substring(3*base,3*base+2)+ "(hex)  " +bits.replaceAll(" ", "0"));
						 System.out.println();

					 }

                 }

		    }
            catch ( IOException e )
            {
                e.printStackTrace();
                System.exit(-1);
            }
        }
    }

    /** */
    public static class SerialWriter implements Runnable
    {

//public boolean write(byte[] data, int offset,int len)

        OutputStream out;
        public SerialWriter ( OutputStream out ){this.out = out;}

        public void run ()
        {
            try
            {
                int c = 0;
                while ( ( c = System.in.read()) > -1 )
                {
					if(c==102 || c==70){//    F(f)
//                                            ====
						System.out.println("Sending to MCU < FF FF FF FF FF FF FF FF >");
						this.out.write(new byte[]{(byte) -1,-1,-1,-1,-1,-1,-1,-1},0,8);
					}

					if(c==114 || c==82){//    R(egsters)
//                                            ==========
						System.out.println("Sending to MCU < 81 >");
						this.out.write(new byte[]{(byte) -127},0,1);
					}

					if(c==103 || c==71){//    G(eneral I/O regsters)
//                                            ======================
//                  <01=memory read> <byte count 0=256> <addHigh 10=16> <addLow 00=0>
						System.out.println("Sending to MCU < G >");
						addHigh=16;
						addLow =0;
						byteSpan=256;
						this.out.write(new byte[]{(byte) 1,0,16,0},0,4);
					}

					if(c==101 || c==69){//    E(eprom)
//                                            ========
//                  <01=memory read> <byte count 0=256> <addHigh FE=-2> <addLow 00=0> then addHigh FF=-1
						System.out.println("Sending to MCU < E >");
						addHigh=Integer.parseInt("FE", 16);
						addLow =0;
						byteSpan=256;
						this.out.write(new byte[]{(byte) 1,0,-2,0},0,4);
                        try {Thread.sleep(1000);} catch (InterruptedException ie) {}
						this.out.write(new byte[]{(byte) 1,0,-1,0},0,4);
					}

					if(c==109 || c==77){//    M(emory)
//                                            ========
//                  <01=memory read> <byte count 0=256> <addHigh 00=0> <addLow 00=0> then addHigh=01, 02, 03
						System.out.println("Sending to MCU < M >");
						addHigh=0;
						addLow =0;
						byteSpan=256;
						this.out.write(new byte[]{(byte) 1,0,0,0},0,4);
                        try {Thread.sleep(1000);} catch (InterruptedException ie) {}
                        this.out.write(new byte[]{(byte) 1,0,1,0},0,4);
                        try {Thread.sleep(1000);} catch (InterruptedException ie) {}
                        this.out.write(new byte[]{(byte) 1,0,2,0},0,4);
                        try {Thread.sleep(1000);} catch (InterruptedException ie) {}
                        this.out.write(new byte[]{(byte) 1,0,3,0},0,4);
                    }

                    if(c==104 || c==72){    // H(elp)
						System.out.println("+-------------------------------------------+");
						System.out.println("|Commands:                                  |");
						System.out.println("|  ^C kill this run                         |");
						System.out.println("|   F to send eight FFs (attempt to resync) |");
						System.out.println("|                                           |");
						System.out.println("|   R dump the MCU registers                |");
						System.out.println("|   G dump all I/O registers {A,B,C,D,E,G}  |");
						System.out.println("|                                           |");
						System.out.println("|   E dump the  512 byte onchip EEPROM      |");
						System.out.println("|   M dump the 1024 byte onchip RAM         |");
						System.out.println("|                                           |");
						System.out.println("|   H display this help message             |");
						System.out.println("+-------------------------------------------+");
						System.out.println();
     			   }
                }
            }
            catch ( IOException e )
            {
                e.printStackTrace();
                System.exit(-1);
            }
        }
    }

    public static void main ( String[] args )
    {
			System.out.println("+-------------------------------------------+");
			System.out.println("|Commands:                                  |");
			System.out.println("|  ^C kill this run                         |");
			System.out.println("|   F to send eight FFs (attempt to resync) |");
			System.out.println("|                                           |");
			System.out.println("|   R dump the MCU registers                |");
			System.out.println("|   G dump all I/O registers {A,B,C,D,E,G}  |");
			System.out.println("|                                           |");
			System.out.println("|   E dump the  512 byte onchip EEPROM      |");
			System.out.println("|   M dump the 1024 byte onchip RAM         |");
			System.out.println("|                                           |");
			System.out.println("|   H display this help message             |");
			System.out.println("+-------------------------------------------+");
			System.out.println();
        try
        {
            (new TalkerJF1()).connect("COM4");
        }
        catch ( Exception e )
        {
            e.printStackTrace();
        }
    }
}